## -------------------------------------------------------------- ##
## Algorithm definitions for real data study
## Called by real-data.R
## -------------------------------------------------------------- ##

## Pec function for simpleRF
predictSurvProb.ForestSurvival <- function(object, newdata, times, ...) {
  pred <- object$predict(newdata)
  pos <- sapply(times, function(x) {
    max(c(0, which(x >= object$timepoints)))
  })    
  p <- cbind(1, pred)[, pos+1, drop = FALSE]
  return(p)
}

## Compute error
compute_error <- function(mod, test) {
  predicted <- mod$predict(test)
  truth <- test$y
  type <- mod$treetype
  
  if (type == "Classification") {
    err <- mean(predicted != truth) # Missclassifications
  } else if (type == "Probability estimation" | type == "Probability") {
    err <- mean((1 - predicted[cbind(truth == "FALSE", truth == "TRUE")])^2) # Brier score
  } else if (type == "Regression") {
    err <- mean((predicted - truth)^2) # MSE
  } else if (type == "Survival") {
    library(ipred)
    times <- sort(unique(truth[, 1]))
    psb <- predictSurvProb.ForestSurvival(mod, test, times)
    err <- sbrier(truth, t(psb), times)[1,1] # Brier score
  } else {
    stop("Unknown tree type.")
  }
  return(err)
}

## simpleRF algo, return prediction error
simpleRF_wrapper <- function(data, job, instance, unordered_factors, ...) {
  library(simpleRF)
  
  ## Shuffle predictor levels
  factor_idx <- sapply(data, is.factor)
  data[, factor_idx] <- lapply(data[, factor_idx], function(x) {
    factor(x, levels = sample(levels(x), nlevels(x)))
  })
  
  ## Subset data
  train <- as.data.frame(data[instance$train, ])
  test <- as.data.frame(data[instance$test, ])
  
  if (unordered_factors == "dummy") {
    ## Change data to dummy coding
    library(dummies)
    dummycols <- setdiff(colnames(train[, sapply(train, is.factor)]), "y")
    train <- dummy.data.frame(train, names = dummycols, drop = FALSE)
    test <- dummy.data.frame(test, names = dummycols, drop = FALSE)

    ## No unordered factor handling required anymore
    unordered_factors <- "ignore"
  }
  
  ## Use max 10 mtry values evenly distributed between 1 and ncol-1
  mtry <- unique(round(seq(1, ncol(train) - 1, length.out = 10)))
  
  ## Tune mtry on training data usign 5-fold CV
  n <- nrow(train)
  folds <- split(sample(n, n), cut(1:n, 5, labels = FALSE))
  tune <- sapply(folds, function(test_idx) {
    train_idx <- setdiff(1:n, test_idx)
    
    sapply(mtry, function(x) {
      mod <- simpleRF(y ~ ., data = train[train_idx, ], mtry = x,
                      unordered_factors = unordered_factors, ...)
      compute_error(mod, train[test_idx, ])
    })
  })
  best_mtry <- mtry[which.min(rowMeans(tune))]
  
  ## Build and evaluate model on whole training dataset
  runtime <- system.time({
    mod <- simpleRF(y ~ ., data = train, mtry = best_mtry,
                    unordered_factors = unordered_factors, ...)
    err <- compute_error(mod, test)
  })
  
  ## Return mse and best mtry
  list(err = err, best_mtry = best_mtry, runtime = runtime["elapsed"])
}
