## -------------------------------------------------------------- ##
## Real data study
## Requires an HPC cluster and the package batchtools
## Expect long runtimes (unfortunately simpleRF is extremely slow!)
## -------------------------------------------------------------- ##

library(ggplot2)
library(xtable)
library(data.table)
library(batchtools)
options(batchtools.verbose = TRUE)
options(stringsAsFactors = FALSE)
source("import-data.R")

set.seed(42)

## Registry
reg_name <- "unordered_real-data"
reg_dir <- file.path("registries", reg_name)
unlink(reg_dir, recursive = TRUE)
reg <- makeExperimentRegistry(file.dir = reg_dir, 
                              source = c("problems.R", "algorithms.R"))

## Algo params (mtry is tuned)
num_trees <- 50

## Simulation params
num_folds <- 10
num_replicates <- 50

## Problems
addProblem(name = "tic_tac_toe", data = tic_tac_toe, fun = crossval, seed = 43)
addProblem(name = "servo", data = servo, fun = crossval, seed = 44)
addProblem(name = "mpg", data = mpg, fun = crossval, seed = 45)
addProblem(name = "ra_snps", data = ra_snps, fun = crossval, seed = 46)
addProblem(name = "splice", data = splice, fun = crossval, seed = 47)
addProblem(name = "aids", data = aids, fun = crossval, seed = 48)

## Algorithms
addAlgorithm(name = "simpleRF", fun = simpleRF_wrapper)

## Experiments
prob_design <- list(tic_tac_toe = expand.grid(num_folds = num_folds,
                                              fold = 1:num_folds),
                    servo = expand.grid(num_folds = num_folds,
                                        fold = 1:num_folds),
                    mpg = expand.grid(num_folds = num_folds,
                                      fold = 1:num_folds),
                    ra_snps = expand.grid(num_folds = num_folds,
                                          fold = 1:num_folds),
                    splice = expand.grid(num_folds = num_folds,
                                         fold = 1:num_folds),
                    aids = expand.grid(num_folds = num_folds,
                                       fold = 1:num_folds))
algo_design <- list(
  simpleRF = expand.grid(num_trees = num_trees, 
                         unordered_factors = c("ignore", "partition", "order_once", "order_split", "dummy"), 
                         stringsAsFactors = FALSE)
)
addExperiments(prob_design, algo_design, repls = num_replicates)

## Remove experiments with impossible combinations (too many levels)
removeExperiments(findExperiments(prob.pattern = "mpg", 
                                  algo.pars = (unordered_factors == "partition")))
summarizeExperiments()

## Test jobs
#testJob(id = 1)

## Submit
if (grepl("node\\d{2}|bipscluster", system("hostname", intern = TRUE))) {
  ids <- findNotStarted()
  ids[, chunk := chunk(job.id, chunk.size = 200)]
  submitJobs(ids = ids, # walltime in seconds, 10 days max, memory in MB
             resources = list(name = reg_name, chunks.as.arrayjobs = TRUE, 
                              ncpus = 1, memory = 6000, walltime = 10*24*3600,
							  max.concurrent.jobs = 200))
} else {
  submitJobs()
}
waitForJobs()

## Get and save results
tab <- flatten(ijoin(getJobTable(), reduceResultsDataTable()))
tab[, mean_err := mean(err), by = list(problem, unordered_factors, repl)]
saveRDS(tab, "real-data.Rds")
#tab <- readRDS("real-data.Rds")

## Rename factor levels
tab$problem <- factor(tab$problem, 
                      levels = c("tic_tac_toe", "splice", "mpg", "servo", "ra_snps", "aids"), 
                      labels = c("Tic-tac-toe", "Splice", "MPG", "Servo", "RA SNPs", "AIDS"))
tab[, unordered_factors := factor(unordered_factors, 
                                  levels = c("partition", "order_split", "order_once", "dummy", "ignore"), 
                                  labels = c("Partition", "Order (split)", "Order (once)", "Dummy", "Ignore"))]

## Compute mean runtimes
tab[, median(runtime), by = list(problem, unordered_factors)]
print(xtable(dcast(data = tab, formula =  problem ~ unordered_factors, 
                   value.var = "runtime", fun.aggregate = median), 
             caption = "Runtime of real data analyses.", 
             label = "tab:real_data_runtime"), 
      booktabs = TRUE, table.placement = "htbp", 
      include.rownames = FALSE)

## Plot runtime
ggplot(tab, aes(x = unordered_factors, y = runtime, fill = unordered_factors)) + 
  geom_boxplot() + 
  scale_y_log10() + 
  facet_wrap(~ problem, scales = "free_y", ncol = 2) + 
  theme(axis.text.x = element_text(angle = 45, hjust = 1)) + 
  xlab("Nominal predictor method") + 
  ylab("Runtime (seconds, logarithmic scale)") + 
  scale_fill_brewer() + 
  theme(legend.position = "none")
ggsave("runtime_real-data.pdf", width = 7, height = 8)

## Plot
ggplot(tab, aes(x = unordered_factors, y = mean_err, fill = unordered_factors)) + 
  facet_wrap(~ problem, scales = "free_y", ncol = 2) + 
  geom_boxplot() + 
  theme(axis.text.x = element_text(angle = 45, hjust = 1)) + 
  xlab("Nominal predictor method") + 
  ylab("Prediction error") + 
  scale_fill_brewer() + 
  theme(legend.position = "none")
ggsave("real-data.pdf", width = 7, height = 8)

# Calculate p-values from corrected t-test, compared to "Partition" method
t.test.corrected <- function(x, y, K, R) {
  # In classification avoid 0's by comparing accuracy
  if (max(c(x, y)) < 1 & min(c(x, y)) == 0) {
    x <- 1 - x
    y <- 1 - y
  }
  dif <- log(x) - log(y)
  t <- mean(dif) / sd(dif) * 1 / sqrt(1 / (K * R) + 1 / (K - 1))
  pval <- 2 * pt(abs(t), df = K * R - 1, lower.tail = FALSE)
  pval
}
tab_w <- dcast(tab, problem + repl + fold ~ unordered_factors, value.var = "err")
tab_w[problem == "MPG", ]$Partition <- tab_w[problem == "MPG", ]$`Order (split)`
pvals <- tab_w[, lapply(.SD, function(x) {t.test.corrected(x = x, y = Partition, K = num_folds, R = num_replicates)}), .SDcols = levels(tab$unordered_factors), by = problem]
print(xtable(pvals[, -2], 
             caption = "p-values of real data.", 
             label = "tab:simulation_pvalues", 
             digits = 3), 
      booktabs = TRUE, table.placement = "htbp", 
      include.rownames = FALSE)
      
      
    