## -------------------------------------------------------------- ##
## Simulation study
## Requires an HPC cluster and the package batchtools
## Expect long runtimes (unfortunately simpleRF is extremely slow!)
## -------------------------------------------------------------- ##

library(ggplot2)
library(xtable)
library(data.table)
library(batchtools)
options(batchtools.verbose = TRUE)
options(stringsAsFactors = FALSE)

set.seed(42)

## Registry
reg_name <- "unordered_simulation"
reg_dir <- file.path("registries", reg_name)
unlink(reg_dir, recursive = TRUE)
reg <- makeExperimentRegistry(file.dir = reg_dir, 
                              source = c("problems.R", "algorithms.R"))
#reg <- loadRegistry(file.dir = reg_dir)

## Data params
n <- c(50, 100, 200)
p <- 4
type <- c("reg", "class", "multiclass", "surv")

## Digit data params
num_levels <- 10

## SNP data params
maf <- .25
beta <- 3 # outcome effect (all but multiclass)

## Group data params (also uses num_levels)
num_groups <- 2

## Algo params (default for mtry and min_node_size)
num_trees <- 50 

## Simulation params
num_replicates <- 100

## Problems
addProblem(name = "digits", fun = digit_data, seed = 43)
addProblem(name = "snps", fun = snp_data, seed = 44)
addProblem(name = "groups", fun = group_data, seed = 45)

## Algorithms
addAlgorithm(name = "simpleRF", fun = simpleRF_wrapper)

## Experiments
prob_design <- list(digits = expand.grid(n = n, p = p, 
                                         num_levels = num_levels, type = type), 
                    snps = expand.grid(n = n, p = p, 
                                       maf = maf, beta = beta, type = type),
                    groups = expand.grid(n = n, p = p, num_groups = num_groups,
                                         num_levels = num_levels, type = type))
algo_design <- list(
  simpleRF = expand.grid(num_trees = num_trees, 
                         probability = FALSE, 
                         unordered_factors = c("ignore", "partition", "order_once", "order_split", "dummy"), 
                         stringsAsFactors = FALSE)
)
addExperiments(prob_design, algo_design, repls = num_replicates)
summarizeExperiments()

## Test jobs
#testJob(id = 1)

## Submit
if (grepl("node\\d{2}|bipscluster", system("hostname", intern = TRUE))) {
  ids <- findNotStarted()
  ids[, chunk := chunk(job.id, chunk.size = 200)]
  submitJobs(ids = ids, # walltime in seconds, 10 days max, memory in MB
             resources = list(name = reg_name, chunks.as.arrayjobs = TRUE, 
				              ncpus = 1, memory = 6000, walltime = 10*24*3600, 
							  max.concurrent.jobs = 200))
} else {
  submitJobs()
}
waitForJobs()

## Get and save results
tab <- flatten(ijoin(reduceResultsDataTable(), getJobPars()))
saveRDS(tab, "simulation.Rds")
#tab <- readRDS("simulation.Rds")

## Rename factor levels
levels(tab$type) <- c("Regression", "Binary Class.", "Multiclass Class.", "Survival")
tab[, problem := factor(problem, 
                        levels = c("digits", "snps", "groups"),
                        labels = c("Digit", "SNP", "Group"))]
tab[, unordered_factors := factor(unordered_factors, 
                                  levels = c("partition", "order_split", "order_once", "dummy", "ignore"), 
                                  labels = c("Partition", "Order (split)", "Order (once)", "Dummy", "Ignore"))]

## Compute mean runtimes (n==100)
tab[, median(runtime), by = list(n, problem, type, unordered_factors)]
print(xtable(dcast(data = tab[n == 100, ], formula = problem + type ~ unordered_factors, 
                   value.var = "runtime", fun.aggregate = median), 
             caption = "Runtime of simulations.", 
             label = "tab:simulation_runtime"), 
      booktabs = TRUE, table.placement = "htbp", 
      include.rownames = FALSE)

## Plot runtime (n==100)
ggplot(tab[n == 100, ], aes(x = unordered_factors, y = runtime, fill = unordered_factors)) + 
  geom_boxplot() + 
  scale_y_log10() + 
  facet_grid(type ~ problem, scales = "free") + 
  theme(axis.text.x = element_text(angle = 45, hjust = 1)) + 
  xlab("Nominal predictor method") + 
  ylab("Runtime (seconds, logarithmic scale)") + 
  scale_fill_brewer() + 
  theme(legend.position = "none")
ggsave("runtime_simulation.pdf", width = 7, height = 10)

## Plot results
lapply(unique(tab$n), function(nn) {
  ggplot(tab[n == nn, ], aes(x = unordered_factors, y = err, fill = unordered_factors)) + 
    geom_boxplot() + 
    facet_grid(type ~ problem, scales = "free_y") + 
    theme(axis.text.x = element_text(angle = 45, hjust = 1)) + 
    xlab("Nominal predictor method") + 
    ylab("Prediction error") + 
    scale_fill_brewer() + 
    theme(legend.position = "none")
  ggsave(paste0("simulation_n", nn, ".pdf"), width = 7, height = 10)
})

# Calculate p-values from t-test, compared to "Partition" method
tab[, repl := rowid(type, problem, n, unordered_factors)]
tab_w <- dcast(tab, problem + type + n + repl ~ unordered_factors, value.var = "err")
tab_w[type == "Multiclass Class.", 5:9] <- 1 - tab_w[type == "Multiclass Class.", 5:9]
tab_w[type == "Binary Class.", 5:9] <- 1 - tab_w[type == "Binary Class.", 5:9]
pvals <- tab_w[, lapply(.SD, function(x) {t.test(log(x) - log(Partition))$p.value}), .SDcols = levels(tab$unordered_factors), by = list(problem, type, n)]
print(xtable(pvals[n == 100, c(-3, -4)],
             caption = "p-values of simulations.",
             label = "tab:simulation_pvalues",
             digits = 1,
             display = c(rep("s", 3), rep("e", 4))),
      booktabs = TRUE, table.placement = "htbp",
      include.rownames = FALSE, math.style.exponents = TRUE)

