## -------------------------------------------------------------- ##
## Problem definitions for censoring simulation study
## Called by simulation_censoring.R
## -------------------------------------------------------------- ##

library(survival)

## Count number of odd digits - number of even digits per row
digit_data <- function(data, job, n, p, num_levels, type = "reg", cens = .3) {
  x_num <- data.frame(replicate(p, sample(num_levels, 2*n, replace = TRUE)))
  x_fac <- data.frame(lapply(x_num, factor, levels = 1:num_levels))
  
  ## +1 for odd, -1 for even
  scores <- data.frame(lapply(x_num, function(x) {
    2*(x %% 2) - 1
  }))
  if (type == "reg") {
    ## Number of odd digits - number of even digits per row
    y <- rowSums(scores)
  } else if (type == "class") {
    ## Number of odd digits > number of even digits per row?
    y <- factor(rowSums(scores) > 0)
  } else if (type == "multiclass") {
    ## One class for each combination of odd/even digits
    y_num <- (scores == 1) %*% 2^seq(0, p-1)
    y <- factor(y_num, levels = sample(unique(y_num)))
  } else if (type == "surv") {
    ## Exponential model for time
    y_linear <- rowSums(scores)
    rs <- -log(runif(2*n, 0, 1))/exp(y_linear)
    time <- rs - min(rs)
    
    ## Random censoring for training data
    status <- c(rbinom(n, 1, 1 - cens), rep(1, n))
    time[1:n] <- sapply(time[1:n], runif, n = 1, min = 0)
    
    y <- Surv(time, status)
  } else {
    stop("Unknown type.")
  }
  
  dat <- data.frame(x_fac, y)
  list(train = dat[1:n, ], test = dat[-(1:n), ])
}

## SNP data with heterozygous advantage
snp_data <- function(data, job, n, p, maf, beta, type = "reg", cens = .3) {
  if (p %% 2 != 0) {
    stop("Only even number of covariates allowed.")
  }
  if (length(beta) == 1) {
    beta <- rep(c(beta, -beta), p/2)
  }
  
  x_num <- data.frame(replicate(p, sample(0:2, 2*n, replace = TRUE, prob = c({1-maf}**2, 2*{1-maf}*maf, maf**2)))) 
  x_fac <- data.frame(lapply(x_num, factor, levels = 0:2))
  
  ## Effect only for x==1 (1 risk allele)
  y_linear <- as.matrix(x_num == 1) %*% matrix(beta, ncol = 1)
  
  if (type == "reg") {
    ## Linear model
    y <- y_linear + rnorm(2*n)
  } else if (type == "class") {
    ## Logit model
    y <- as.factor(rbinom(2*n, size = 1, prob = plogis(y_linear)))
  } else if (type == "multiclass") {
    ## One class for each combination of heterozygous SNPs
    y_num <- (x_num == 1) %*% 2^seq(0, p-1)
    y <- factor(y_num, levels = sample(unique(y_num)))
  } else if (type == "surv") {
    ## Exponential model for time
    rs <- -log(runif(2*n, 0, 1))/exp(y_linear)
    time <- rs - min(rs)
    
    ## Random censoring for training data
    status <- c(rbinom(n, 1, 1 - cens), rep(1, n))
    time[1:n] <- sapply(time[1:n], runif, n = 1, min = 0)
    
    y <- Surv(time, status)
  } else {
    stop("Unknown type.")
  }
  
  dat <- data.frame(x_fac, y)
  list(train = dat[1:n, ], test = dat[-(1:n), ])
}

## Group data: Different relationships in groups
group_data <- function(data, job, n, p, num_levels, type = "reg", cens = .3, num_groups) {
  ## Generate multinomial data without dummy coding
  rmultinomnumeric <- function(n, size, prob) {
    dummy <- rmultinom(n, size, prob)
    apply(dummy, 2, function(x) which(x == 1))
  }
  
  x_num <- data.frame(replicate(p, rmultinomnumeric(2*n, 1, rep(1, num_levels))))
  x_fac <- data.frame(lapply(x_num, factor, levels = 1:num_levels))
  
  ## Group assignment
  groups <- rmultinomnumeric(2*n, 1, rep(1, num_groups))
  
  # Linear effects with shuffled factors in groups
  scores <- as.matrix(replicate(p, rep(NA, 2*n)))
  for (j in 1:p) {
    beta <- replicate(num_groups, sample(1:num_levels))
    for (i in 1:(2*n)) {
      scores[i, j] <- beta[x_num[i, j], groups[i]]
    }
  }
  
  # Result is dependent on outcome
  if (type == "reg") {
    ## Combined effect
    y <- rowMeans(scores)
  } else if (type == "class") {
    ## Outcome > median outcome?
    y <- factor(rowMeans(scores) > median(rowMeans(scores)))
  } else if (type == "multiclass") {
    ## One class for each combination of high/low
    y_num <- (scores > num_levels/2) %*% 2^seq(0, p-1)
        y <- factor(y_num, levels = sample(unique(y_num)))
  } else if (type == "surv") {
    ## Exponential model for time
    y_linear <- rowMeans(scores)
    rs <- -log(runif(2*n, 0, 1))/exp(y_linear)
    time <- rs - min(rs)
    
    ## Random censoring for training data
    status <- c(rbinom(n, 1, 1 - cens), rep(1, n))
    time[1:n] <- sapply(time[1:n], runif, n = 1, min = 0)
    
    y <- Surv(time, status)
  } else {
    stop("Unknown type.")
  }
  
  dat <- data.frame(groups, x_fac, y)
  list(train = dat[1:n, ], test = dat[-(1:n), ])
}

