## -------------------------------------------------------------- ##
## Simulation study for varying censoring proportion
## Requires an HPC cluster and the package batchtools
## Expect long runtimes (unfortunately simpleRF is extremely slow!)
## -------------------------------------------------------------- ##

library(ggplot2)
library(xtable)
library(data.table)
library(batchtools)
options(batchtools.verbose = TRUE)
options(stringsAsFactors = FALSE)

set.seed(42)

## Registry
reg_name <- "unordered_simulation"
reg_dir <- file.path("registries", reg_name)
unlink(reg_dir, recursive = TRUE)
reg <- makeExperimentRegistry(file.dir = reg_dir, 
                              source = c("problems.R", "algorithms.R"))
#reg <- loadRegistry(file.dir = reg_dir, writeable = TRUE)

## Data params
n <- 100
p <- 4
type <- "surv"
cens <- seq(0, .9, by = .1)

## Digit data params
num_levels <- 10

## SNP data params
maf <- .25
beta <- 3 # outcome effect (all but multiclass)

## Group data params (also uses num_levels)
num_groups <- 2

## Algo params (default for mtry and min_node_size)
num_trees <- 50 

## Simulation params
num_replicates <- 100

## Problems
addProblem(name = "digits", fun = digit_data, seed = 43)
addProblem(name = "snps", fun = snp_data, seed = 44)
addProblem(name = "groups", fun = group_data, seed = 45)

## Algorithms
addAlgorithm(name = "simpleRF", fun = simpleRF_wrapper)

## Experiments
prob_design <- list(digits = expand.grid(n = n, p = p, cens = cens, 
                                         num_levels = num_levels, type = type), 
                    snps = expand.grid(n = n, p = p, cens = cens,
                                       maf = maf, beta = beta, type = type), 
                    groups = expand.grid(n = n, p = p, cens = cens, num_groups = num_groups,
                                         num_levels = num_levels, type = type))
algo_design <- list(
  simpleRF = expand.grid(num_trees = num_trees, 
                         probability = FALSE, 
                         unordered_factors = c("ignore", "partition", "order_once", "order_split", "dummy"), 
                         stringsAsFactors = FALSE)
)
addExperiments(prob_design, algo_design, repls = num_replicates)
summarizeExperiments()

## Test jobs
#testJob(id = 1)

## Submit
if (grepl("node\\d{2}|bipscluster", system("hostname", intern = TRUE))) {
  ids <- findNotStarted()
  ids[, chunk := chunk(job.id, chunk.size = 200)]
  submitJobs(ids = ids, # walltime in seconds, 10 days max, memory in MB
             resources = list(name = reg_name, chunks.as.arrayjobs = TRUE, 
				              ncpus = 1, memory = 6000, walltime = 10*24*3600, 
							  max.concurrent.jobs = 200))
} else {
  submitJobs()
}
waitForJobs()

## Get and save results
tab <- flatten(ijoin(reduceResultsDataTable(), getJobPars()))
saveRDS(tab, "simulation_censoring.Rds")
#tab <- readRDS("simulation_censoring.Rds")

# ## Compute mean runtimes
# tab[, median(runtime), by = list(n, problem, type, unordered_factors)]
# print(xtable(dcast(data = tab, formula = n + problem + type ~ unordered_factors, 
#                    value.var = "runtime", fun.aggregate = median), 
#              caption = "Runtime of simulations.", 
#              label = "tab:simulation_runtime"), 
#       booktabs = TRUE, table.placement = "htbp", 
#       include.rownames = FALSE)

## Plot results
tab[, problem := factor(problem, 
                        levels = c("digits", "snps", "groups"),
                        labels = c("Digit", "SNP", "Group"))]
tab[, unordered_factors := factor(unordered_factors, 
                        levels = c("partition", "order_split", "order_once", "dummy", "ignore"), 
                        labels = c("Partition", "Order (split)", "Order (once)", "Dummy", "Ignore"))]
ggplot(tab, aes(x = cens, y = err, col = unordered_factors, shape = unordered_factors)) + 
  stat_summary(fun.y = median, geom = "point") + 
  stat_summary(fun.y = median, geom = "line") + 
  facet_wrap( ~ problem) + 
  xlab("Censoring proportion") + 
  ylab("Prediction error") + 
  theme(legend.position = "top") + 
  scale_colour_brewer("Nominal predictor method", palette = "Set1") + 
  scale_shape_manual("Nominal predictor method", values = 1:5) 
ggsave("simulation_censoring.pdf", width = 7, height = 3.5)


